//$Id: PlayerDataContainer.h,v 1.6 2002/03/18 09:46:58 yoshi Exp $
/*============================================================================

  player data container class
        copyright (C) 2001,2002 HOSHINO Yoshifumi,(Yoshi)

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


============================================================================*/
#ifndef _CLASS_PLAYER_DATA_CONTAINER
#define _CLASS_PLAYER_DATA_CONTAINER

#define WIN32_LEAN_AND_MEAN

#include <windows.h>
#include <shellapi.h>

#define MAX_TITLE_SIZE 2048

#define GWT_REFRESH  TRUE
#define GWT_COPYONLY FALSE

#define GPS_STOPPED 0
#define GPS_PLAYING 1
#define GPS_PAUSED  3
#define GPS_MISSING 9

#define GPT_ELAPSED          0
#define GPT_REMAIN           1
#define GPT_TOTAL            10
#define GPT_ELAPSED_MILLISEC 20




/******************************************************************************
  Define Class
******************************************************************************/
class CPlayerDataContainer
{
public:
  //*******************************************************
  //constructor
  CPlayerDataContainer(LPSTR ClassName=NULL, LPSTR DefaultStr =NULL, LPSTR DefaultPath=NULL) {
    m_lpszTitle        = new char[MAX_TITLE_SIZE];
    m_lpszTitleCache   = new char[MAX_TITLE_SIZE];
    m_lpszDefaultTitle = new char[MAX_TITLE_SIZE];
    m_lpszWndClassName = new char[256];
    m_lpszPath         = new char[MAX_PATH];

    ZeroMemory(m_lpszTitle,        MAX_TITLE_SIZE);
    ZeroMemory(m_lpszTitleCache,   MAX_TITLE_SIZE);
    ZeroMemory(m_lpszDefaultTitle, MAX_TITLE_SIZE);
    ZeroMemory(m_lpszWndClassName, 256);
    ZeroMemory(m_lpszPath,         MAX_PATH);

    m_bUseServer = TRUE;

    //windowclassname of target
    if(ClassName)
      strncpy(m_lpszWndClassName, ClassName, 256);

    //default string for missing player.
    if(DefaultStr)
      strncpy(m_lpszDefaultTitle, DefaultStr, MAX_TITLE_SIZE);
    else
      strcpy(m_lpszDefaultTitle, "PlayerDataContainer");

    //default string for missing path.
    if(DefaultPath)
      strncpy(m_lpszPath, DefaultPath, MAX_TITLE_SIZE);

    m_dwLastCommand=0;
    m_uiStatus=GPS_MISSING;
    m_uiPlaybackTime=0;
    m_uiPlaybackLength=0;
  }
  //*******************************************************
  //destructor
  ~CPlayerDataContainer(){
    if(m_lpszTitle)
      delete m_lpszTitle;
    if(m_lpszDefaultTitle)
      delete m_lpszDefaultTitle;
    if(m_lpszPath)
      delete m_lpszPath;
    if(m_lpszWndClassName)
      delete m_lpszWndClassName;
  }

  //*******************************************************
  virtual BOOL ExecuteEx(HWND owner, BOOL bShown){
    char path[MAX_PATH];
    if(GetPath(path)){
      ShellExecute(owner, "open", path,NULL, NULL, (bShown)?SW_SHOW:SW_MINIMIZE);
      return TRUE;
    }
    return FALSE;
  }

  //******************************************************
  virtual BOOL Close(){
    HWND hwnd= FindWindow();
    if(hwnd){
      PostMessage(hwnd, WM_CLOSE, 0, 0);
      return TRUE;
    }
    return FALSE;
  }

  //*******************************************************
  virtual void GetWindowTitle(LPSTR TitleBuffer,size_t bufsize) {
    if(FindWindow())
      strncpy(TitleBuffer, m_lpszTitle, bufsize-1);
    else
      strncpy(TitleBuffer, m_lpszDefaultTitle, bufsize-1);
  }

  //*******************************************************
  virtual long GetPlaybackTime(DWORD mode){
    if(m_uiPlaybackTime == -1 || m_uiPlaybackLength == -1)
      return 0;

    if(mode == GPT_ELAPSED)
      return m_uiPlaybackTime/1000;
    if(mode == GPT_REMAIN)
      return (m_uiPlaybackLength - m_uiPlaybackTime)/1000;
    if(mode == GPT_TOTAL)
      return m_uiPlaybackLength /1000;
    if(mode == GPT_ELAPSED_MILLISEC)
      return m_uiPlaybackTime;
    return 0;
  }

  //*******************************************************
  virtual HWND FindWindow() {
    if(m_lpszWndClassName)
      return ::FindWindow(m_lpszWndClassName,NULL);

    return NULL;
  }

  //*******************************************************
  virtual void DisplayMainMenu() {
    HWND hwnd = FindWindow();

    if(hwnd){
      SetForegroundWindow(hwnd);
      Sleep(100);
      PostMessage(hwnd, 0x313, 0, GetMessagePos());
    }
    return;
  }

  //*******************************************************
  virtual void SetPlayerType(int version) { return; }
  virtual UINT GetPlaybackStatus() { return m_uiStatus; }
  virtual BOOL PostCommand(DWORD command) { m_dwLastCommand=command; return TRUE;}
  virtual DWORD GetLastCommand() { DWORD data = m_dwLastCommand; m_dwLastCommand=0; return data; }
  virtual void SetFormat(LPSTR TitleBuffer, BOOL Prefix, BOOL Suffix) { return; }
  virtual BOOL GetPath(LPSTR path) { strcpy(path, m_lpszPath); return TRUE; }
  virtual void SetPath(LPSTR path) { strcpy(m_lpszPath, path); return;}

  //*******************************************************
  BOOL GetTitle(LPSTR TitleBuffer, BOOL refresh) {
    if(!TitleBuffer || !m_lpszTitle)
      return FALSE;

    GetWindowTitle(TitleBuffer, MAX_TITLE_SIZE);

    if(strcmp(m_lpszTitleCache, TitleBuffer)){
      strcpy(m_lpszTitleCache, TitleBuffer);
      return TRUE;
    }
    return FALSE;
  }

  //data input ********************************************
  void SetTitle(LPSTR TitleBuffer) { strncpy(m_lpszTitle, TitleBuffer, MAX_TITLE_SIZE-1); }
  void SetPlaybackTime(long total,long elapsed) {
    m_uiPlaybackLength = total;
    m_uiPlaybackTime = elapsed;
  }
  void SetPlaybackStatus(UINT ps) {m_uiStatus = ps; }

  //*******************************************************
  //Legacy Functions
  BOOL Execute(HWND owner){ return ExecuteEx(owner, FALSE);}

  //*******************************************************
  BOOL m_bUseServer;
protected:
  DWORD m_dwLastCommand;

  LPSTR m_lpszWndClassName;
  LPSTR m_lpszTitle;
  LPSTR m_lpszTitleCache;
  LPSTR m_lpszDefaultTitle;
  LPSTR m_lpszPath;

  UINT m_uiStatus;
  long m_uiPlaybackTime;
  long m_uiPlaybackLength;
};

#endif  //_CLASS_PLAYER_DATA_CONTAINER
